@@ -16,7 +16,7 @@ use tui::backend::CrosstermBackend;
 use tui::layout::{Constraint, Direction, Layout, Rect};
 use tui::style::{Color, Modifier, Style};
 use tui::text::{Span, Spans};
-use tui::widgets::{Block, Borders, Gauge, Tabs};
+use tui::widgets::{Block, Borders, Tabs};
 use tui::Frame;
 use tui::Terminal;
 use tui_logger::*;
@@ -29,7 +29,6 @@ pub struct TuiApp {
     states: Vec<TuiWidgetState>,
     tabs: Vec<String>,
     selected_tab: usize,
-    opt_info_cnt: Option<u16>,
 }
 
 #[derive(Debug)]
@@ -47,6 +46,7 @@ pub enum TuiAppEvent {
     HideKey,
     FocusKey,
     TabKey,
+    BackTabKey,
 }
 
 impl TuiApp {
@@ -56,7 +56,6 @@ impl TuiApp {
             states: vec![],
             tabs: vec![],
             selected_tab: 0,
-            opt_info_cnt: None,
         }
     }
     pub fn init(tx: mpsc::Sender<event::Event>) -> BoxResult<Self> {
@@ -121,6 +120,12 @@ impl TuiApp {
                         KeyCode::Char('f') => {
                             tx.send(TuiApp(FocusKey)).unwrap();
                         }
+                        KeyCode::Tab => {
+                            tx.send(TuiApp(TabKey)).unwrap();
+                        }
+                        KeyCode::BackTab => {
+                            tx.send(TuiApp(BackTabKey)).unwrap();
+                        }
                         _ => {}
                     }
                 }
@@ -130,9 +135,11 @@ impl TuiApp {
         Ok(TuiApp {
             terminal: Some(terminal),
             states: vec![],
-            tabs: vec!["V1".into()],
+            tabs: vec!["1", "2", "3", "4"]
+                .into_iter()
+                .map(|t| t.into())
+                .collect(),
             selected_tab: 0,
-            opt_info_cnt: None,
         })
     }
     pub fn deinit(&mut self) -> BoxResult<()> {
@@ -163,7 +170,14 @@ impl TuiApp {
             MinusKey => Some(TuiWidgetEvent::MinusKey),
             HideKey => Some(TuiWidgetEvent::HideKey),
             FocusKey => Some(TuiWidgetEvent::FocusKey),
-            TabKey => None,
+            TabKey => {
+                self.selected_tab = (self.selected_tab + 1) % self.tabs.len();
+                None
+            }
+            BackTabKey => {
+                self.selected_tab = (self.selected_tab + self.tabs.len() - 1) % self.tabs.len();
+                None
+            }
         };
         if let Some(widget_evt) = widget_evt {
             self.states[self.selected_tab].transition(&widget_evt);
@@ -188,26 +202,16 @@ fn draw_frame<B: Backend>(t: &mut Frame<B>, size: Rect, app: &mut TuiApp) {
         .collect();
     let sel = app.selected_tab;
 
-    if app.states.len() <= sel {
-        app.states.push(TuiWidgetState::new());
+    while app.states.len() <= sel {
+        app.states
+            .push(TuiWidgetState::new().set_default_display_level(LevelFilter::Info));
     }
 
-    let block = Block::default().borders(Borders::ALL);
-    let inner_area = block.inner(size);
-    t.render_widget(block, size);
-
-    let mut constraints = vec![
-        Constraint::Length(3),
-        Constraint::Percentage(50),
-        Constraint::Min(3),
-    ];
-    if app.opt_info_cnt.is_some() {
-        constraints.push(Constraint::Length(3));
-    }
+    let constraints = vec![Constraint::Length(3), Constraint::Min(3)];
     let chunks = Layout::default()
         .direction(Direction::Vertical)
         .constraints(constraints)
-        .split(inner_area);
+        .split(size);
 
     let tabs = Tabs::new(tabs)
         .block(Block::default().borders(Borders::ALL))
@@ -229,31 +233,4 @@ fn draw_frame<B: Backend>(t: &mut Frame<B>, size: Rect, app: &mut TuiApp) {
         .output_line(true)
         .state(&app.states[sel]);
     t.render_widget(tui_sm, chunks[1]);
-    let tui_w: TuiLoggerWidget = TuiLoggerWidget::default()
-        .block(
-            Block::default()
-                .title("Independent Tui Logger View")
-                .border_style(Style::default().fg(Color::White).bg(Color::Black))
-                .borders(Borders::ALL),
-        )
-        .output_separator('|')
-        .output_timestamp(Some("%F %H:%M:%S%.3f".to_string()))
-        .output_level(Some(TuiLoggerLevelOutput::Long))
-        .output_target(false)
-        .output_file(false)
-        .output_line(false)
-        .style(Style::default().fg(Color::White).bg(Color::Black));
-    t.render_widget(tui_w, chunks[2]);
-    if let Some(percent) = app.opt_info_cnt {
-        let gauge = Gauge::default()
-            .block(Block::default().borders(Borders::ALL).title("Progress"))
-            .gauge_style(
-                Style::default()
-                    .fg(Color::Black)
-                    .bg(Color::White)
-                    .add_modifier(Modifier::ITALIC),
-            )
-            .percent(percent);
-        t.render_widget(gauge, chunks[3]);
-    }
 }
